/*
 * Copyright 2013-2018 Radian Memory Systems, Inc.  All Rights Reserved.
 *
 * This file is provided under non-disclosure agreement.
 * See the file COPYING for details.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include "nvme_express.h"
#include "nvme_ioctl.h"
typedef __u64 u64;
typedef __u32 u32;
typedef __u16 u16;
typedef __u8 u8;
#define MAX_DIES 256
#include "RMSVendorSpecific.h"
#include "nvme_util.h"

#define FW_STORE 0
#define FW_STORE_ACTIVATE 1
#define FW_ACTIVATE 2

#define CONTROL_DEV		"/dev/nvme0"

static void
wait_for_q(void)
{
	system("/bin/stty raw");
	getchar();
	system("/bin/stty cooked");
}

static void
create_activate_cmd(
	struct usr_io *x,
	unsigned long slot)
{
	memset(x, 0x0, sizeof(struct usr_io));
	x->cmd.header.opCode = NVM_ADMIN_CMD_FIRMWARE_ACTIVATE;
	x->direction = XFER_TO_DEV;
	x->timeout = 1;
	x->cmd.cmd.firmwareActivate.action = FW_ACTIVATE;
	x->cmd.cmd.firmwareActivate.slot = slot;
}

static void
usage(void)
{
	printf("Select the firmware version to load the next time the\n");
	printf("board is reset.\n");
	printf("usage: firmware_activate [devName] <slot_number>\n");
	printf("   devName: The device to be updated - default /dev/nvme0\n");
	printf("   <slot_number> must between 1 and 4 inclusive\n");
}

int
main(int argc, char **argv)
{
	struct usr_io x;
	int fd = -1;
	int status;
	int slot;
	char *control_dev = CONTROL_DEV;

	if (argc == 2) {
		if (strcmp(argv[1], "-h") == 0) {
			usage();
			exit(1);
		}
		if (strcmp(argv[1], "-H") == 0) {
			usage();
			wait_for_q();
			exit(1);
		}
	}
	if (argc == 3) {
		/* Override default device */
		if (strncmp("/dev/", argv[1], 5) == 0)
			control_dev = argv[1];
		slot = atoi(argv[2]);
	} else {
		if (argc != 2) {
			usage();
			exit(1);
		}
		slot = atoi(argv[1]);
	}

	if (slot < 1 || slot > 4) {
		usage();
		exit(1);
	}

	fd = open(control_dev, O_RDWR);
	if (fd < 0) {
		printf("Could not open file %s ", control_dev);
		if (errno == EACCES)
			printf(" Access error - try running as su\n");
		else
			printf(" Error %d\n", errno);
		exit(1);
	}

	create_activate_cmd(&x, slot);
	printf("activate slot %d\n", slot);
	status = ioctl(fd, NVME_IOCTL_ADMIN_CMD, &x);
	printf("status = %d\n", status);
	if (x.comp.SCT != 0 || x.comp.SC != 0) {
		/*
		* For the following combination - it's not realy an error
		*/
		if (x.comp.SCT == 1 && x.comp.SC == 0x0b) {
			printf("%s\n",
				nvme_err(x.comp.SCT, x.comp.SC));
		} else {
			printf("Operation failed, NVMe error = %s\n",
				nvme_err(x.comp.SCT, x.comp.SC));
		}
	} else {
		printf("OK!\n");
	}

	close(fd);
	exit(0);
}
