/*****************************************************************************
 *                Copyright (C) 2013 Radian Memory Systems
 *
 * This file is provided under non-disclosure agreement.
 * See the file COPYING for details.
 ******************************************************************************/
/*
 * Look at SCT and SC from the NVMe completion struct and translate the
 * error code into a string.
 */


char *generic_error[] = {
	"OK",
	"Invalid command opcode",
	"Invalid field in command",
	"Command ID conflict",
	"Data Transfer Error",
	"Commands Aborted Due to Power Loss Notification",
	"Internal Device Error",
	"Command Abort Requested",
	"Command Aborted due to SQ Deletion",
	"Command Aborted due to Failed Fused Command",
	"Command Aborted due to Missing Fused Command",
	"Invalid Namespace or Format",
};

char *command_specific_error[] = {
	"Completion Queue Invalid",
	"Invalid Queue Identifier",
	"Maximum Queue Size Exceeded",
	"Abort Command Limit Exceeded",
	"Requested Command to Abort Not Found",
	"Asynchronous Event Request Limit",
	"Invalid Firmware Slot",
	"Invalid Firmware Image",
	"Invalid Interrupt Vector",
	"Invalid Log Page",
	"Invalid Format",
	"Firmware Application Requires Conventional Reset",
	"Invalid Queue Deletion",
	"Snapshot Out of Range",
};

char *media_error[] = {
	"Write Fault",
	"Unrecoverable Read Error",
	"End-to-End Guard Check Error",
	"End-to-End Application Tag Check Error",
	"End-to-End Reference Tag Check Error",
	"Compare Failure",
	"Access Denied"
};

char *
nvme_err(unsigned int status_code_type, unsigned int status_code)
{
	switch (status_code_type) {
	case 0:  /* Generic error */
		if (status_code < 0xc)
			return generic_error[status_code];
		else if (status_code == 0x80)
			return "LBA Out of Range";
		else if (status_code == 0x81)
			return "Capacity Exceeded";
		else if (status_code == 0x82)
			return "Namespace Not Ready";
		break;
	case 1:  /* Command-specific error */
		if (status_code < 0xe)
			return command_specific_error[status_code];
		else if (status_code == 0x80)
			return "Conflicting Attributes";
		break;

	case 2:  /* Media error */
		if (status_code >= 0x80 && status_code <= 0x86)
			return media_error[status_code - 0x80];
		break;
	default:
		break;
	}
	return "unknown";
}

